@@ -1,5 +1,5 @@
-## Mon May 27 03:39:04 2019
-## Original file Copyright © 2019 A.C. Guidoum, K. Boukhetala
+## Fri Apr 03 08:54:53 2020
+## Original file Copyright © 2020 A.C. Guidoum, K. Boukhetala
 ## This file is part of the R package Sim.DiffProc
 ## Department of Probabilities & Statistics
 ## Faculty of Mathematics
@@ -27,25 +27,24 @@
 #####
 ##### PredCorr1D
 
-.PredCorr1D <- function(N =1000,M=1,x0=0,t0=0,T=1,Dt=NULL,alpha=0.5,mu=0.5,
+.PredCorr1D <- function(N =1000,M=1,x0=0,t0=0,T=1,Dt,alpha=0.5,mu=0.5,
                                drift,diffusion,type=c("ito","str"),...)
                        {
     DSx  <- Simplify(Deriv(diffusion,"x",cache.exp=FALSE)) 
     if (type=="ito"){A    <- function(t,x)  eval(drift)}else{
-	driftstr <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = drift[[1]], e2 = diffusion[[1]], de2 = Deriv(diffusion,"x",cache.exp=FALSE)[[1]]))))
+	driftstr <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = drift[[1]], e2 = diffusion[[1]], de2 = Deriv(diffusion,"x",cache.exp=FALSE)[[1]]))))
 	A  <- function(t,x) eval(driftstr)
-    # A <- function(t,x) eval(drift) - 0.5 * eval(diffusion) * eval(DSx)
 	}
     S    <- function(t,x)  eval(diffusion)
     Sx   <- function(t,x)  eval(DSx)
     SS   <- function(t,x)  A(t,x) - mu * S(t,x) * Sx(t,x)
-    if (is.null(Dt)) {
-        Dt <- (T - t0)/N
+    if (missing(Dt)) {
         t <- seq(t0, T, by=Dt)
     } else {
         t <- c(t0, t0 + cumsum(rep(Dt, N)))
 		T <- t[N + 1]
     }
+    Dt <- (T - t0)/N
     Wu <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
 	Wd <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
     X = XX <- matrix(x0, N+1, M)	
@@ -65,35 +64,33 @@
 #####
 ##### PredCorr2D
 
-.PredCorr2D <- function(N =1000,M=1,x0=0,y0=0,t0=0,T=1,Dt=NULL,alpha=0.5,mu=0.5,driftx,
+.PredCorr2D <- function(N =1000,M=1,x0=0,y0=0,t0=0,T=1,Dt,alpha=0.5,mu=0.5,driftx,
                         diffx,drifty,diffy,type=c("ito","str"),...)
-                       {
+ {
     DSx  <- Simplify(Deriv(diffx,"x",cache.exp=FALSE))
     DSy  <- Simplify(Deriv(diffy,"y",cache.exp=FALSE))
     if (type=="ito"){
     Ax    <- function(t,x,y)  eval(driftx)
     Ay    <- function(t,x,y)  eval(drifty)
     }else{
-	driftstrx <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]]))))
-	driftstry <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]]))))
+	driftstrx <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]]))))
+	driftstry <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]]))))
     Ax <- function(t,x,y) eval(driftstrx) 
     Ay <- function(t,x,y) eval(driftstry)
-    # Ax <- function(t,x,y) eval(driftx) - 0.5 * eval(diffx) * eval(DSx)
-    # Ay <- function(t,x,y) eval(drifty) - 0.5 * eval(diffy) * eval(DSy)
          }
     Sx    <- function(t,x,y)  eval(diffx)
     dSx   <- function(t,x,y)  eval(DSx)
     SSx   <- function(t,x,y)  Ax(t,x,y) - mu * Sx(t,x,y) * dSx(t,x,y)
     Sy    <- function(t,x,y)  eval(diffy)
     dSy   <- function(t,x,y)  eval(DSy)
-    SSy   <- function(t,x,y)  Ay(t,x,y) - mu * Sy(t,x,y) * dSy(t,x,y)	
-    if (is.null(Dt)) {
-        Dt <- (T - t0)/N
+    SSy   <- function(t,x,y)  Ay(t,x,y) - mu * Sy(t,x,y) * dSy(t,x,y)
+    if (missing(Dt)) {
         t <- seq(t0, T, by=Dt)
     } else {
         t <- c(t0, t0 + cumsum(rep(Dt, N)))
 		T <- t[N + 1]
     }
+    Dt <- (T - t0)/N	
     Wux <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
 	Wdx <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
     Wuy <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
@@ -116,7 +113,7 @@
 #####
 ##### PredCorr3D
 
-.PredCorr3D <- function(N =1000,M=1,x0=0,y0=0,z0=0,t0=0,T=1,Dt=NULL,alpha=0.5,mu=0.5,driftx,diffx,drifty,diffy,
+.PredCorr3D <- function(N =1000,M=1,x0=0,y0=0,z0=0,t0=0,T=1,Dt,alpha=0.5,mu=0.5,driftx,diffx,drifty,diffy,
                      driftz,diffz,type=c("ito","str"),...)
                        {
     DSx  <- Simplify(Deriv(diffx,"x",cache.exp=FALSE))
@@ -127,15 +124,12 @@
     Ay    <- function(t,x,y,z)  eval(drifty)
     Az    <- function(t,x,y,z)  eval(driftz)
     }else{
-	driftstrx <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]]))))
-	driftstry <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]]))))
-    driftstrz <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = driftz[[1]], e2 = diffz[[1]], de2 = Deriv(diffz,"z",cache.exp=FALSE)[[1]]))))
+	driftstrx <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]]))))
+	driftstry <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]]))))
+    driftstrz <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = driftz[[1]], e2 = diffz[[1]], de2 = Deriv(diffz,"z",cache.exp=FALSE)[[1]]))))
     Ax <- function(t,x,y,z) eval(driftstrx) 
     Ay <- function(t,x,y,z) eval(driftstry)
     Az <- function(t,x,y,z) eval(driftstrz)
-    # Ax <- function(t,x,y,z) eval(driftx) - 0.5 * eval(diffx) * eval(DSx)
-    # Ay <- function(t,x,y,z) eval(drifty) - 0.5 * eval(diffy) * eval(DSy)
-    # Az <- function(t,x,y,z) eval(driftz) - 0.5 * eval(diffz) * eval(DSz)
          }
     Sx    <- function(t,x,y,z)  eval(diffx)
     dSx   <- function(t,x,y,z)  eval(DSx)
@@ -146,13 +140,13 @@
     Sz    <- function(t,x,y,z)  eval(diffz)
     dSz   <- function(t,x,y,z)  eval(DSz)
     SSz   <- function(t,x,y,z)  Az(t,x,y,z) - mu * Sz(t,x,y,z) * dSz(t,x,y,z)
-    if (is.null(Dt)) {
-        Dt <- (T - t0)/N
+    if (missing(Dt)) {
         t <- seq(t0, T, by=Dt)
     } else {
         t <- c(t0, t0 + cumsum(rep(Dt, N)))
 		T <- t[N + 1]
-    }
+    } 
+    Dt <- (T - t0)/N
     Wux <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
 	Wdx <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
     Wuy <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
