@@ -1,5 +1,5 @@
-## Mon May 27 03:39:04 2019
-## Original file Copyright © 2019 A.C. Guidoum, K. Boukhetala
+## Thu Apr 30 10:50:58 2020
+## Original file Copyright © 2020 A.C. Guidoum, K. Boukhetala
 ## This file is part of the R package Sim.DiffProc
 ## Department of Probabilities & Statistics
 ## Faculty of Mathematics
@@ -48,6 +48,26 @@ TEX.sde.default <- function(object, ...)
    greek8 <- c(paste0(greek,"8"))
    greek9 <- c(paste0(greek,"9"))
    greek10 <- c(paste0(greek,"10"))
+   greek11 <- c(paste0(greek,"11"))
+   greek12 <- c(paste0(greek,"12"))
+   greek13 <- c(paste0(greek,"13"))
+   greek14 <- c(paste0(greek,"14"))
+   greek15 <- c(paste0(greek,"15"))
+   greek16 <- c(paste0(greek,"16"))
+   greek17 <- c(paste0(greek,"17"))
+   greek18 <- c(paste0(greek,"18"))
+   greek19 <- c(paste0(greek,"19"))
+   greek20 <- c(paste0(greek,"20"))
+   greek21 <- c(paste0(greek,"21"))
+   greek22 <- c(paste0(greek,"22"))
+   greek23 <- c(paste0(greek,"23"))
+   greek24 <- c(paste0(greek,"24"))
+   greek25 <- c(paste0(greek,"25"))
+   greek26 <- c(paste0(greek,"26"))
+   greek27 <- c(paste0(greek,"27"))
+   greek28 <- c(paste0(greek,"28"))
+   greek29 <- c(paste0(greek,"29"))
+   greek30 <- c(paste0(greek,"30"))
    greek_list  <- setNames(paste0("\\", greek),greek)
    greek_list0 <- setNames(paste0("\\", greek,"_","{0}"), greek0)
    greek_list1 <- setNames(paste0("\\", greek,"_","{1}"), greek1)
@@ -60,6 +80,26 @@ TEX.sde.default <- function(object, ...)
    greek_list8 <- setNames(paste0("\\", greek,"_","{8}"), greek8)
    greek_list9 <- setNames(paste0("\\", greek,"_","{9}"), greek9)
    greek_list10 <- setNames(paste0("\\", greek,"_","{10}"), greek10)
+   greek_list11 <- setNames(paste0("\\", greek,"_","{11}"), greek11)
+   greek_list12 <- setNames(paste0("\\", greek,"_","{12}"), greek12)
+   greek_list13 <- setNames(paste0("\\", greek,"_","{13}"), greek13)
+   greek_list14 <- setNames(paste0("\\", greek,"_","{14}"), greek14)
+   greek_list15 <- setNames(paste0("\\", greek,"_","{15}"), greek15)
+   greek_list16 <- setNames(paste0("\\", greek,"_","{16}"), greek16)
+   greek_list17 <- setNames(paste0("\\", greek,"_","{17}"), greek17)
+   greek_list18 <- setNames(paste0("\\", greek,"_","{18}"), greek18)
+   greek_list19 <- setNames(paste0("\\", greek,"_","{19}"), greek19)
+   greek_list20 <- setNames(paste0("\\", greek,"_","{20}"), greek20)
+   greek_list21 <- setNames(paste0("\\", greek,"_","{21}"), greek21)
+   greek_list22 <- setNames(paste0("\\", greek,"_","{22}"), greek22)
+   greek_list23 <- setNames(paste0("\\", greek,"_","{23}"), greek23)
+   greek_list24 <- setNames(paste0("\\", greek,"_","{24}"), greek24)
+   greek_list25 <- setNames(paste0("\\", greek,"_","{25}"), greek25)
+   greek_list26 <- setNames(paste0("\\", greek,"_","{26}"), greek26)
+   greek_list27 <- setNames(paste0("\\", greek,"_","{27}"), greek27)
+   greek_list28 <- setNames(paste0("\\", greek,"_","{28}"), greek28)
+   greek_list29 <- setNames(paste0("\\", greek,"_","{29}"), greek29)
+   greek_list30 <- setNames(paste0("\\", greek,"_","{30}"), greek30)
    mem_sde <- c("m","S","m1","m2","m3","S1","S2","S3","C12","C13","C23")
    mem_sde_tex <- c("m(t)","S(t)","m_{1}(t)","m_{2}(t)","m_{3}(t)","S_{1}(t)","S_{2}(t)","S_{3}(t)","C_{12}(t)","C_{13}(t)","C_{23}(t)")
    mem_sde_list <- setNames(mem_sde_tex, mem_sde)
@@ -72,7 +112,10 @@ TEX.sde.default <- function(object, ...)
    #fct_sde_tex <- c("\\mathbb{P}")
    #fct_sde_list <- setNames(fct_sde_tex, fct_sde)
    greek_env   <- list2env(as.list(c(greek_list,greek_list0,greek_list1,greek_list2,greek_list3,greek_list4,greek_list5,
-                                     greek_list6,greek_list7,greek_list8,greek_list9,greek_list10,
+                                     greek_list6,greek_list7,greek_list8,greek_list9,greek_list10,greek_list11,greek_list12,
+									 greek_list13,greek_list14,greek_list15,greek_list16,greek_list17,greek_list18,greek_list19,
+									 greek_list20,greek_list21,greek_list22,greek_list23,greek_list24,greek_list25,greek_list26,
+									 greek_list27,greek_list28,greek_list29,greek_list30,
                                      mem_sde_list,var_sde_list)), parent = emptyenv())
    unary_op <- function(left, right) {
            force(left)
@@ -121,8 +164,10 @@ TEX.sde.default <- function(object, ...)
    f_env$cos  <- unary_op("\\cos(", ")")
    f_env$tan  <- unary_op("\\tan(", ")")
    f_env$exp  <- unary_op("\\exp(", ")")
+   f_env$expm1  <- unary_op("\\left(\\exp(", ")-1\\right)")
    f_env$sqrt <- unary_op("\\sqrt{", "}")
    f_env$log  <- unary_op("\\log(", ")")
+   f_env$log1p  <- unary_op("\\log\\left(1+","\\right)")
    f_env$abs  <- unary_op("\\left| ", "\\right| ")
    f_env$sign  <- unary_op("\\mathop{\\mathrm{sgn}}(", ")")
    f_env$"/"  <- function(a, b) {
